@@ -30,6 +30,7 @@ using System.Threading;
 using NDesk.DBus;
 using org.freedesktop.DBus;
 using org.freedesktop.Telepathy;
+using Mono.Unix;
 
 namespace Banter
 {
@@ -63,6 +64,10 @@ namespace Banter
 		protected ConnectionInfo connInfo;
 		protected IConnection tlpConnection;
 		protected IConnectionManager connManager;
+		protected IChannelGroup localInvitationGroup;
+		protected IChannelGroup remoteInvitationGroup;
+		protected IChannelGroup subscribedGroup;
+		protected IChannelGroup blockedGroup;
 
 		// Connected Handlers
 		protected bool aliasConnected = false;
@@ -107,7 +112,13 @@ namespace Banter
 		
 		public string BusName
 		{
-			get {return busName;}
+			get
+			{
+				try {
+					return connInfo.BusName;
+				} catch{}
+				return busName;
+			}
 		}
 		
 		public ObjectPath BusPath
@@ -172,6 +183,52 @@ namespace Banter
 		{
 			get {return tlpConnection;}
 		}
+
+		public string[] ReceivedInvitations
+		{
+			get
+			{
+				try {
+					return 
+						tlpConnection.InspectHandles (
+							HandleType.Contact,
+							localInvitationGroup.LocalPendingMembers);
+				} catch{}
+				return null;
+			}
+		}
+		
+		public uint[] ReceivedInvitationHandles
+		{
+			get
+			{
+				try {return localInvitationGroup.LocalPendingMembers;} catch{}
+				return null;
+			}
+		}
+		
+		public string[] SentInvitations
+		{
+			get
+			{
+				try {
+					return 
+						tlpConnection.InspectHandles (
+							HandleType.Contact,
+							remoteInvitationGroup.RemotePendingMembers);
+				} catch{}
+				return null;
+			}
+		}
+		
+		public uint[] SentInvitationHandles
+		{
+			get
+			{
+				try {return remoteInvitationGroup.RemotePendingMembers;} catch{}
+				return null;
+			}
+		}
 		
 		#region Constructors
 		public Account ()
@@ -183,6 +240,14 @@ namespace Banter
 		{
 			this.protocol = protocol;
 		}
+		
+		/// <summary>
+		/// Construct using an existing telepathy connection
+		/// </summary>
+		public Account (IConnection connection)
+		{
+			tlpConnection = connection;
+		}
 		#endregion
 
 		/// <summary>
@@ -192,6 +257,7 @@ namespace Banter
 		/// </summary>
 		protected void SetupSupportedInterfaces ()
 		{
+			Logger.Debug ("SetupSupportedInterfaces - called");
 			foreach (string iname in tlpConnection.Interfaces)
 			{
 				switch (iname)
@@ -224,6 +290,104 @@ namespace Banter
 			}
 		}
 
+		/// <summary>
+		/// Method to open and maintain outstanding channels
+		/// to the publish and subscribe lists
+		/// </summary>
+		protected void SetupGroupChannels ()
+		{
+			Logger.Debug ("SetupGroupChannels - called");
+			
+			string[] pubArgs = {"publish"};
+			string[] subArgs = {"subscribe"};
+			string[] blockedArgs = {"deny"};
+			ObjectPath objectPath;
+			
+			try {
+				// First setup a channel to the publish list
+				uint[] pubHandles = tlpConnection.RequestHandles (HandleType.List, pubArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						pubHandles[0], 
+						true);
+						
+				localInvitationGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+				localInvitationGroup.MembersChanged += OnPublishedMembersChanged;
+			
+				/*
+				public enum ChannelGroupFlag : uint
+				{
+					CanAdd = 1,
+					CanRemove = 2,
+					CanRescind = 4,
+					MessageAdd = 8,
+					MessageRemove = 16,
+					MessageAccept = 32,
+					MessageReject = 64,
+					MessageRescind = 128,
+					SpecificHandles = 256
+				}
+				*/
+				
+				
+				uint groupFlags = localInvitationGroup.GroupFlags;
+				Logger.Debug ("Group Flags: {0}", groupFlags);
+				if ((((uint) groupFlags & (uint) ChannelGroupFlag.CanAdd) == (uint) ChannelGroupFlag.CanAdd))
+					Logger.Debug ("  can add");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.CanRemove) == (uint) ChannelGroupFlag.CanRemove)
+					Logger.Debug ("  can remove");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.CanRescind) == (uint) ChannelGroupFlag.CanRescind)
+					Logger.Debug ("  can rescind");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageAdd) == (uint) ChannelGroupFlag.MessageAdd)
+					Logger.Debug ("  can add messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageRemove) == (uint) ChannelGroupFlag.MessageRemove)
+					Logger.Debug ("  can remove messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageAccept) == (uint) ChannelGroupFlag.MessageAccept)
+					Logger.Debug ("  can accept messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageReject) == (uint) ChannelGroupFlag.MessageReject)
+					Logger.Debug ("  can reject messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageRescind) == (uint) ChannelGroupFlag.MessageRescind)
+					Logger.Debug ("  can rescind messages");
+				// Next setup a channel to the subscribe list
+				uint[] subHandles = tlpConnection.RequestHandles (HandleType.List, subArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						subHandles[0], 
+						true);
+						
+				remoteInvitationGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+					
+				subscribedGroup = remoteInvitationGroup;
+				subscribedGroup.MembersChanged += OnSubscribedMembersChanged;
+				
+				// Next setup a channel to the blocked list
+				uint[] blockedHandles = 
+					tlpConnection.RequestHandles (HandleType.List, blockedArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						blockedHandles[0], 
+						true);
+						
+				blockedGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+					
+				blockedGroup.MembersChanged += OnBlockedMembersChanged;
+				
+			} catch (Exception sgc) {
+				Logger.Debug ("Failed setting up group channels");
+				Logger.Debug (sgc.Message);
+				Logger.Debug (sgc.StackTrace);
+			}
+		}
+
 		/// <summary>
 		/// Telepathy callback when state is changing on the connection
 		/// </summary>
@@ -245,6 +409,7 @@ namespace Banter
 					try
 					{
 						SetupSupportedInterfaces ();
+						SetupGroupChannels ();
 						SetupMe ();
 						SetupProviderUsers ();
 						AdvertiseCapabilities ();
@@ -285,16 +450,21 @@ namespace Banter
 		
 		protected void SetupMe ()
 		{
+			Logger.Debug ("SetupMe - called");
 			string[] aliasNames = null;
+			uint[] meHandles = { tlpConnection.SelfHandle };
 	
 			try {
+				// Get my capabilities
+				CapabilityInfo[] caps = tlpConnection.GetCapabilities (meHandles);
+				foreach (CapabilityInfo cap in caps)
+					Logger.Debug ("Caps - channel type: {0}", cap.ChannelType);
+				
+				// Request my alias
+				if (aliasing == true)
+					aliasNames = tlpConnection.RequestAliases (meHandles);
 
 				string uri = options["account"] as string;
-				if (aliasing == true) {
-					uint[] ids = { tlpConnection.SelfHandle };
-					aliasNames = tlpConnection.RequestAliases (ids);
-				}	
-
 				string meKey = ProviderUserManager.CreateKey (uri, protocol);
 				ProviderUser me = null;				
 				try {
@@ -331,44 +501,117 @@ namespace Banter
 				Logger.Debug (sm.StackTrace);
 			}
 		}
-		
+
 		protected void SetupProviderUsers ()
 		{
-			//string[] args = {"subscribe"};
-			string[] args = {"known"};
-			uint[] memberHandles = tlpConnection.RequestHandles (HandleType.List, args);
-			ObjectPath op = 
-				tlpConnection.RequestChannel (
-					org.freedesktop.Telepathy.ChannelType.ContactList, 
-					HandleType.List, 
-					memberHandles[0], 
-					true);
+			Logger.Debug ("SetupProviderUsers - called");
+			ProviderUser providerUser;
+			string key;
+			
+			// First add any sent invitation users
+			Logger.Debug ("Adding SentInvitation users");
+			uint[] invitedHandles = this.SentInvitationHandles;
+			if (invitedHandles != null && invitedHandles.Length > 0) {
+				string[] invitedUsers = this.SentInvitations;
+				
+				for (int i = 0; i < invitedUsers.Length; i++) {
 					
-			Logger.Debug ("# known contacts: {0}", memberHandles.Length);
+					Logger.Debug ("  {0}", invitedUsers[i]);
 					
-			IChannelGroup cl = Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, op);
-			string[] members = tlpConnection.InspectHandles (HandleType.Contact, cl.Members);
+					// update the provider user objects
+					key = ProviderUserManager.CreateKey (invitedUsers[i], protocol);
+					providerUser = null;
+					try {
+						providerUser = ProviderUserManager.GetProviderUser (key);
+						providerUser.TlpConnection = tlpConnection;
+						providerUser.AccountName = this.Name;
+						providerUser.Protocol = this.Protocol;
+						providerUser.ID = invitedHandles[i];
+						providerUser.Relationship = ProviderUserRelationship.SentInvitation;
+					} 
+					catch (Exception fff) {
+						Logger.Debug ("Failed to get ProviderUser {0}", key);
+						Logger.Debug (fff.Message);
+					}
+						
+					if (providerUser == null) {
+						try {
+							providerUser =
+								ProviderUserManager.CreateProviderUser (
+									invitedUsers[i], 
+									this.protocol, 
+									ProviderUserRelationship.SentInvitation);
+							providerUser.TlpConnection = tlpConnection;
+							providerUser.AccountName = this.Name;
+							providerUser.ID = invitedHandles[i];
+						} catch{}
+					}
+				}
+			}
+			
+			// Next add received invitation users
+			uint[] inviteHandles = this.ReceivedInvitationHandles;
+			if (inviteHandles != null && inviteHandles.Length > 0) {
+				string[] inviteUsers = this.ReceivedInvitations;
+				
+				for (int i = 0; i < inviteUsers.Length; i++) {
+					
+					// update the provider user objects
+					key = ProviderUserManager.CreateKey (inviteUsers[i], protocol);
+							
+					providerUser = null;
+					try {
+						providerUser = ProviderUserManager.GetProviderUser (key);
+						providerUser.TlpConnection = tlpConnection;
+						providerUser.AccountName = this.Name;
+						providerUser.Protocol = this.Protocol;
+						providerUser.ID = inviteHandles[i];
+						providerUser.Relationship = ProviderUserRelationship.ReceivedInvitation;
+					} 
+					catch (Exception fff) {
+						Logger.Debug ("Failed to get ProviderUser {0}", key);
+						Logger.Debug (fff.Message);
+					}
+						
+					if (providerUser == null) {
+						try {
+							providerUser =
+								ProviderUserManager.CreateProviderUser (
+									inviteUsers[i], 
+									this.protocol, 
+									ProviderUserRelationship.ReceivedInvitation);
+							providerUser.AccountName = this.Name;
+							providerUser.TlpConnection = tlpConnection;
+							providerUser.ID = inviteHandles[i];
+						} catch{}
+					}
+				}
+			}
+			
+			// Add subscribed members			
+			string[] members = 
+				tlpConnection.InspectHandles (HandleType.Contact, subscribedGroup.Members);
 			
 			string[] aliasNames = null;
 			if (aliasing == true) {
-				aliasNames = tlpConnection.RequestAliases (cl.Members);
-				Logger.Debug ("# returned aliases: {0}", aliasNames.Length);	
+				aliasNames = tlpConnection.RequestAliases (subscribedGroup.Members);
+				//Logger.Debug ("# returned aliases: {0}", aliasNames.Length);	
 			}	
 
-			Logger.Debug ("# of known contacts: {0}", members.Length);
+			//Logger.Debug ("# of known contacts: {0}", members.Length);
 			for (int i = 0; i < members.Length; i++) {
-				Logger.Debug ("MemberID: {0} Member: {1}", cl.Members[i], members[i]);
+				//Logger.Debug ("MemberID: {0} Member: {1}", cl.Members[i], members[i]);
 				
 				// update the provider user objects
-				string key = ProviderUserManager.CreateKey (members[i], protocol);
-						
-				ProviderUser providerUser = null;
+				key = ProviderUserManager.CreateKey (members[i], protocol);
+				providerUser = null;
 				try {
 					providerUser = ProviderUserManager.GetProviderUser (key);
 					providerUser.TlpConnection = tlpConnection;
 					providerUser.AccountName = this.Name;
 					providerUser.Protocol = this.Protocol;
-					providerUser.ID = cl.Members[i];
+					providerUser.ID = subscribedGroup.Members[i];
+					providerUser.Relationship = ProviderUserRelationship.Linked;
 					if (aliasing == true && aliasNames != null)
 						providerUser.Alias = aliasNames[i];
 				} 
@@ -379,26 +622,30 @@ namespace Banter
 					
 				if (providerUser == null) {
 					try {
-						providerUser = new ProviderUser (tlpConnection, cl.Members[i]);
+						providerUser =
+							ProviderUserManager.CreateProviderUser (
+								members[i], 
+								this.protocol, 
+								ProviderUserRelationship.Linked);
+								
 						providerUser.AccountName = this.Name;
-						providerUser.Protocol = this.Protocol;
-						providerUser.Uri = members[i];
+						providerUser.ID = subscribedGroup.Members[i];
+						providerUser.TlpConnection = tlpConnection;
+					
 						if (aliasing == true && aliasNames != null)
 							providerUser.Alias = aliasNames[i];
-						
-						ProviderUserManager.AddProviderUser (key, providerUser);
 					} catch{}
 				}
 			}
 			
 			ConnectHandlers ();
 			if (presence == true)
-				tlpConnection.RequestPresence (cl.Members);
+				tlpConnection.RequestPresence (subscribedGroup.Members);
 		}
 		
 		private void ConnectHandlers ()
 		{
-			Logger.Debug ("Account::ConnectHandlers");
+			//Logger.Debug ("Account::ConnectHandlers");
 			
 			this.tlpConnection.NewChannel += OnNewChannel;
 			channelConnected = true;
@@ -421,7 +668,7 @@ namespace Banter
 		
 		private void DisconnectHandlers ()
 		{
-			Logger.Debug ("Account::DisconnectHandlers");
+			//Logger.Debug ("Account::DisconnectHandlers");
 			
 			try {
 				if (avatarsConnected == true) {
@@ -483,8 +730,6 @@ namespace Banter
 		private void UpdatePresence (ProviderUser user, string presence, string message)
 		{
 			Banter.Presence banterPresence = new Banter.Presence (Banter.PresenceType.Offline);
-			if (message != null && message != String.Empty)
-				banterPresence.Message = message;
 				
 			switch (presence)
 			{
@@ -510,12 +755,16 @@ namespace Banter
 					break;
 			}
 
-			Logger.Debug (
+			// Set the message after the type so the message is saved off
+			if (message != null && message != String.Empty)
+				banterPresence.Message = message;
+
+/*			Logger.Debug (
 				"Updating presence for: {0} to {1}:{2}", 
 				user.Uri, 
 				banterPresence.Type.ToString(), 
 				banterPresence.Message);
-				
+*/				
 			user.Presence = banterPresence;
 		}
 	
@@ -544,6 +793,230 @@ namespace Banter
 			}
 		}
 		
+
+		/// <summary>
+		/// Callback method when members are added or removed from
+		/// the blocked list
+		/// </summary>
+		private
+		void
+		OnBlockedMembersChanged(
+			string message,
+			uint[] added,
+			uint[] removed,
+			uint[] localPending,
+			uint[] remotePending,
+			uint actor,
+			uint reason)
+		{
+			Logger.Debug ("OnBlockedMembersChanged - called");
+			Logger.Debug ("  Message: {0}", message);
+			Logger.Debug ("  # added: {0}", added.Length);
+			foreach (uint handle in added)
+			{
+				Logger.Debug ("    contact id: {0}", handle);
+				
+				/*
+				ProviderUser user =
+					ProviderUserManager.GetProviderUser (handle);
+				user.Relationship = ProviderUserRelationship.Linked;
+				*/
+			}
+				
+			Logger.Debug ("  # removed: {0}", removed.Length);
+			foreach (uint handle in removed) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("    contact id: {0} - {1}", handles[0], names[0]);
+			}
+				
+			Logger.Debug ("Remote Pending List");
+			foreach (uint handle in remotePending) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("   {0} - {1}", handle, names[0]);
+			}
+		}
+		
+		/// <summary>
+		/// Callback method when members are added or removed from
+		/// the subscribed list
+		/// </summary>
+		private
+		void
+		OnSubscribedMembersChanged(
+			string message,
+			uint[] added,
+			uint[] removed,
+			uint[] localPending,
+			uint[] remotePending,
+			uint actor,
+			uint reason)
+		{
+			Logger.Debug ("OnSubscribedMembersChanged - called");
+			if (message != null && message != String.Empty)
+				Logger.Debug ("  Message: {0}", message);
+				
+			try {
+				Logger.Debug ("  # added: {0}", added.Length);
+				
+				if (added.Length > 0) {
+					// Add newly added members
+					string[] aliasNames = null;
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, added);
+			
+					for (int i = 0; i < added.Length; i++) {
+						Logger.Debug (
+							" adding contact id: {0}  name: {1}", 
+							added[i], 
+							names[i]);
+					
+						ProviderUser user =
+							ProviderUserManager.GetProviderUser (added[i]);
+						if (user != null) {
+							user.Relationship = ProviderUserRelationship.Linked;
+							if (aliasNames != null && 
+								aliasNames.Length >= i+1 &&
+								aliasNames[i] != null &&
+								aliasNames[i] != String.Empty)
+									user.Alias = aliasNames[i];
+						}	
+					}
+				}
+				
+				Logger.Debug ("  # removed: {0}", removed.Length);
+				if (removed.Length > 0) {
+					for (int i = 0; i < removed.Length; i++) {
+						Logger.Debug ("    contact id: {0}", removed[i]);
+						ProviderUserManager.RemoveProviderUser (removed[i], this.protocol);
+					}
+				}
+
+				Logger.Debug ("Remote Pending List");
+				if (remotePending.Length > 0) {
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, remotePending);
+						
+					for (int i = 0; i < names.Length; i++) {
+						Logger.Debug ("    contact id: {0} - {1}", remotePending[i], names[i]);
+					}
+				}
+				
+				/*
+				foreach (uint handle in remotePending) {
+					uint[] handles = {handle};
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, handles);
+					Logger.Debug ("   {0} - {1}", handle, names[0]);
+					//this.remoteInvitationGroup.RemoveMembers (handles, String.Empty);
+				}
+				*/
+			} catch (Exception osmc) {
+				Logger.Debug (osmc.Message);
+				Logger.Debug (osmc.StackTrace);
+			}
+		}
+		
+
+		/// <summary>
+		/// Callback method when members are added or removed from
+		/// the published list
+		/// </summary>
+		private
+		void
+		OnPublishedMembersChanged(
+			string message,
+			uint[] added,
+			uint[] removed,
+			uint[] localPending,
+			uint[] remotePending,
+			uint actor,
+			uint reason)
+		{
+			Logger.Debug ("OnPublishedMembersChanged - called");
+			Logger.Debug ("  Message: {0}", message);
+			
+			try {
+				Logger.Debug ("  # added: {0}", added.Length);
+				
+				if (added.Length > 0) {
+					// Add newly added members
+					string[] aliasNames = null;
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, added);
+			
+					for (int i = 0; i < added.Length; i++) {
+						Logger.Debug (
+							" adding contact id: {0}  name: {1}", 
+							added[i], 
+							names[i]);
+					
+						ProviderUser user =
+							ProviderUserManager.GetProviderUser (added[i]);
+						if (user != null) {
+							user.Relationship = ProviderUserRelationship.Linked;
+							if (aliasNames != null && 
+								aliasNames.Length >= i+1 &&
+								aliasNames[i] != null &&
+								aliasNames[i] != String.Empty)
+									user.Alias = aliasNames[i];
+						}	
+					}
+				}
+			} catch (Exception ae) {
+				Logger.Debug (ae.Message);			
+			}
+				
+			if (removed.Length > 0) {
+				try {
+					Logger.Debug ("  # removed: {0}", removed.Length);
+					for (int i = 0; i < removed.Length; i++) {
+						Logger.Debug ("    contact id: {0}", removed[i]);
+						//ProviderUserManager.RemoveProviderUser (removed[i], this.protocol);
+					}
+					
+				} catch (Exception re) {
+					Logger.Debug (re.Message);
+				}
+			}
+				
+			Logger.Debug ("Local Pending List");
+			foreach (uint handle in localPending) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("   {0} - {1}", handle, names[0]);
+				
+				// Add him to the list	
+				ProviderUser providerUser =				
+					ProviderUserManager.CreateProviderUser (
+						names[0], 
+						this.protocol, 
+						ProviderUserRelationship.ReceivedInvitation);
+				providerUser.TlpConnection = tlpConnection;
+				providerUser.AccountName = this.Name;
+				providerUser.ID = handle;
+				
+				if (aliasing == true) {
+					string[] aliasNames = null;
+					aliasNames = tlpConnection.RequestAliases (handles);
+					if (aliasNames != null && aliasNames.Length >= 1)
+						providerUser.Alias = aliasNames[0];
+				}
+			}
+			
+			Logger.Debug ("Remote Pending List");
+			foreach (uint handle in remotePending) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("   {0} - {1}", handle, names[0]);
+			}
+		}
+		
 		/// <summary>
 		/// Telepathy callback when new channels are created
 		/// </summary>
@@ -554,163 +1027,159 @@ namespace Banter
 						uint handle,
 						bool suppressHandler)
 		{	
+			Logger.Debug ("Account::OnNewChannel - called");
 			Logger.Debug ("New Channel {0}", channelType);
 			Logger.Debug ("Handle Type: {0}", handleType.ToString());
 			Logger.Debug ("Handle: {0}", handle);
 			Logger.Debug ("Suppress Handler: {0}", suppressHandler);
 			
-			Conversation conversation = null;
-			switch (channelType)
-			{
-				case org.freedesktop.Telepathy.ChannelType.Text:
-				{
-					if (handle == 0)
-						return;
-						
-					// Check if we have an existing conversation with the peer user
-					ProviderUser pu = null;
-					try {
-						pu = ProviderUserManager.GetProviderUser (handle);
-					} catch{}
-					
-					if (pu == null) return;
-					
-					if (ConversationManager.Exist (pu) == true) {
-						Logger.Debug ("An existing conversation with {0} already exists", pu.Uri);
-						return;
-					}
-					
-					Person peer = PersonManager.GetPersonByJabberId (pu.Uri);
-					ChatWindow cw = null;
-					
-					Logger.Debug ("Peer: {0}", peer.Id);
-					Logger.Debug ("Peer Name: {0}", peer.EDSContact.GivenName);
-					
-					if (ChatWindow.AlreadyExist (peer.Id) == true) { 
-						Logger.Debug ("ChatWindow already exists with this peer");
-						ChatWindow.PresentWindow (peer.Id);
-					} else {
-						try
-						{
-							Logger.Debug ("creating conversation object");
-							conversation = ConversationManager.Create (this, peer, false);
-							IChannelText txtChannel = 
-								Bus.Session.GetObject<IChannelText> (busName, channelPath);
-							
-							conversation.SetTextChannel (txtChannel);
-							Logger.Debug ("created new conversation object");
-						
-							cw = new ChatWindow (conversation);
-							cw.Present();
-						}
-						catch (Exception es)
-						{
-							Logger.Debug (es.Message);
-							Logger.Debug (es.StackTrace);
-						}
-					}
-					break;
-				}
+			if (suppressHandler == false)
+				ConversationManager.ProcessNewChannel (
+					this,
+					channelPath,
+					channelType,
+					handleType,
+					handle,
+					suppressHandler);
 				
-				case org.freedesktop.Telepathy.ChannelType.StreamedMedia:
-				{
-					IChannelStreamedMedia ichannel = 
-						Bus.Session.GetObject<IChannelStreamedMedia> (
-							busName,
-							channelPath);
+			return;
+		}
 		
-					if (handle == 0) {
-					
-						if (ichannel.LocalPendingMembers.Length > 0) {
-							Logger.Debug ("Incoming media conversation");
-							handle = ichannel.LocalPendingMembers[0];
-						} else if (ichannel.RemotePendingMembers.Length > 0) {
-							handle = ichannel.RemotePendingMembers[0];
-							Logger.Debug ("Pulled the handle from ichannel.RemotePendingMembers");
-							return;
-						} else if (ichannel.Members.Length > 0) {
-							handle = ichannel.Members[0];
-							Logger.Debug ("Pulled the handle from ichannel.Members");
-							return;
-						} else {
-							Logger.Debug ("Could not resolve the remote handle");
-							return;
-						}	
-					} else {
-						Logger.Debug ("Handle was non-zero {0} - returning", handle);
-						return;
-					}
-					
-					if (handle == this.tlpConnection.SelfHandle) {
-						Logger.Debug ("Handle was me - yay");
-						uint[] meHandles = {handle};
-						
-						uint[] ids = {ichannel.Members[0]};
-							
-						// Check if we have an existing conversation with the peer user
-						ProviderUser pu = null;
-						try {
-							pu = ProviderUserManager.GetProviderUser (handle);
-						} catch{}
-					
-						if (pu == null) return;
-					
-						if (ConversationManager.Exist (pu) == true) {
-							Logger.Debug ("An existing conversation with {0} already exists", pu.Uri);
-							return;
-						}
-						
-						ichannel.AddMembers(meHandles, String.Empty);
-					
-						Person peer = PersonManager.GetPersonByJabberId (pu.Uri);
-						ChatWindow cw = null;
-					
-						Logger.Debug ("Peer: {0}", peer.Id);
-						Logger.Debug ("Peer Name: {0}", peer.EDSContact.GivenName);
-					
-						if (ChatWindow.AlreadyExist (peer.Id) == true) { 
-							Logger.Debug ("ChatWindow already exists with this peer");
-							ChatWindow.PresentWindow (peer.Id);
-						} else {
-							try
-							{
-								Logger.Debug ("creating conversation object");
-								conversation = ConversationManager.Create (this, peer, false);
-								IChannelText txtChannel = 
-									Bus.Session.GetObject<IChannelText> (busName, channelPath);
-							
-								conversation.SetTextChannel (txtChannel);
-								conversation.SetMediaChannel (ichannel, channelPath);
-								Logger.Debug ("created new conversation object");
-						
-								cw = new ChatWindow (conversation);
-								cw.Present();
-							}
-							catch (Exception es)
-							{
-								Logger.Debug (es.Message);
-								Logger.Debug (es.StackTrace);
-							}
-						}
-					}
-					
-					/*
-					Contact contact = contact_list.ContactLookup (ids[0]);
-					if (contact == null)
-						return;
-						
-					StreamChannel stream_channel = new StreamChannel (this, ichannel, contact, ServiceName, channel);	
-					if ((stream_channel != null) && (!channel_list.Contains (stream_channel)))
-						channel_list.Add (stream_channel);
-
-					if (ChannelCreated != null)
-						ChannelCreated (this, (Channel) stream_channel);
-					*/
-					break;
-				}
+		/// <summary>
+		/// Method to add a member
+		/// </summary>
+		public void AddMember (uint id, string message)
+		{
+			if (this.localInvitationGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
 				
-				default:
-					break;
+			uint[] ids = {id};
+			this.localInvitationGroup.AddMembers (ids, message);	
+		}
+
+
+		/// <summary>
+		/// Method to authorize a user who is requesting it to
+		/// enable chat.  
+		/// authorize == false denies the user of authorization
+		/// </summary>
+		public void AuthorizeUser (bool authorize, uint id, string message)
+		{
+			if (this.localInvitationGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+				
+			uint[] ids = {id};
+			if (authorize == true) {
+				localInvitationGroup.AddMembers (ids, message);
+			}	
+			else
+				localInvitationGroup.RemoveMembers (ids, message);
+		}
+
+		/// <summary>
+		/// Method to block an existing user
+		/// block == true blocks the user, false unblocks
+		/// </summary>
+		public void BlockUser (bool block, uint id, string message)
+		{
+			if (this.blockedGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+				
+			uint[] ids = {id};
+			if (block == true) {
+				subscribedGroup.RemoveMembers (ids, message);
+				blockedGroup.AddMembers (ids, message);
+			} else {
+				blockedGroup.RemoveMembers (ids, message);
+				subscribedGroup.AddMembers (ids, message);
+			}	
+		}
+
+
+		/// <summary>
+		/// Method to invite a user to enable chat
+		/// </summary>
+		public void InviteUser (string username)
+		{
+			Logger.Debug ("InviteUser - called");
+			
+			if (this.remoteInvitationGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+			
+			try {
+				string[] names = {username};
+				uint[] inviteHandles =
+					tlpConnection.RequestHandles (HandleType.Contact, names);
+				
+				remoteInvitationGroup.AddMembers (inviteHandles, String.Empty);
+				
+				string[] normalizedNames = 
+					tlpConnection.InspectHandles (HandleType.Contact, inviteHandles);
+
+				ProviderUser providerUser =				
+					ProviderUserManager.CreateProviderUser (
+						normalizedNames[0], 
+						this.protocol, 
+						ProviderUserRelationship.SentInvitation);
+				providerUser.TlpConnection = tlpConnection;
+				providerUser.AccountName = this.Name;
+				providerUser.ID = inviteHandles[0];
+				
+			} catch (Exception iu ) {
+				Logger.Debug (iu.Message);
+				Logger.Debug (iu.StackTrace);
+			}
+		}
+
+		/// <summary>
+		/// Method to remove an existing user
+		/// </summary>
+		public void RemoveUser (uint id, string message)
+		{
+			Logger.Debug ("Account::RemoveUser - called");
+			Logger.Debug ("  removing: {0}", id);
+
+			if (this.tlpConnection == null)
+				throw new ApplicationException ("Invalid telepathy connection");
+				
+			/*			
+			if (this.subscribedGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+			*/	
+				
+//			string[] subArgs = {"subscribe"};
+			string[] subArgs = {"known"};
+			ObjectPath objectPath;
+			IChannelGroup subGroup;
+			
+			try {
+				uint[] subHandles = tlpConnection.RequestHandles (HandleType.List, subArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						subHandles[0], 
+						true);
+						
+				subGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+
+				Logger.Debug ("Members in \"Known\" group");					
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, subGroup.Members);
+				for (int i = 0; i < subGroup.Members.Length; i++) {
+					Logger.Debug ("Handle: {0}  Name: {1}", subGroup.Members[i], names[i]);
+				}
+					
+				subGroup.MembersChanged += OnSubscribedMembersChanged;
+				uint[] ids = {id};
+				subGroup.RemoveMembers (ids, message);				
+				subGroup.MembersChanged -= OnSubscribedMembersChanged;
+				subGroup = null;
+				
+			} catch (Exception ru) {
+				Logger.Debug ("Exception in RemoveUser");
+				Logger.Debug (ru.Message);
 			}
 		}
 
@@ -736,6 +1205,8 @@ namespace Banter
 			Logger.Debug ("Account::Disconnect - called");
 			if (connected == true && tlpConnection != null) {
 				try {
+					//localInvitationGroup.Close ();
+					//remoteInvitationGroup.Close ();
 					//DisconnectHandlers ();
 					Logger.Debug ("Calling telepathy disconnect");
 					tlpConnection.Disconnect ();
@@ -785,6 +1256,10 @@ namespace Banter
 		}
 		
 		#region Constructors
+		public JabberAccount (IConnection connection) : base (connection)
+		{
+		}
+		
 		public JabberAccount (
 				//TelepathyProvider provider,
 				string accountname,
@@ -819,7 +1294,6 @@ namespace Banter
 			options.Add ("ignore-ssl-errors", true);
 			
 			TelepathyConnectionSetup ();
-			//TapiocaSetup();
 		}
 		#endregion
 		
